%%
%The following script is a tutorial on the Empirical Likelihood-ratio 
%Test with a mean-variance constraint which we call ELTSeq. It provides a 
%differential expression analysis between 2 groups of an RNA-seq experiment.
%As input, the ELTSeq expects raw read count data, 
%as obtained in the form of a rectangular table of integer values. 
%The table cell in the i-th row and the j-th column of the table tells how 
%many reads have been mapped to gene i in sample j.
%A subset of the prostate cancer data which was analyzed in the paper was 
%used here as sample input.
%The RNA-seq data was obtained by sequencing the transcriptome (polyA+) of 
%20 prostate cancer tumors and 10 matched norm tissues using the Illumina 
%GAII platform. We started from raw sequence reads and performed read
%mapping through Bowtie2 and summarized the read counts. As the sample
%data, it only contains randomly selected 1000 genes' read count of all 30 
%samples. It takes 3 minutes for ELTSeq to run a DE analysis of the example 
%dataset on a desktop with one Intel Xeon CPU E5-1620 3.6GHz.

%% Reading in the data
%read in gene read count table of tumor samples, the first column of the
%table is gene names.
tumor_count=importdata('T_count_table');
%n1=number of tumor samples 
n1=size(tumor_count.data,2);
%read in gene read count table of normal samples, the first column of the
%table is gene names. 
normal_count=importdata('N_count_table');
%n2=number of normal samples
n2=size(normal_count.data,2);
%gene names are stored in tumor_count.rowheaders

%combine read count tables together. 
readcount_table=[tumor_count.data normal_count.data];
gene_name=tumor_count.rowheaders;
%% ELTSeq
%intial values and settings that will be used in the fmincon function
x0=[1/n1*ones(n1,1);1/n2*ones(n2,1)];
lb=zeros(n1+n2,1);
ub=ones(n1+n2,1);
beq=[1;1;0];
options = optimoptions(@fmincon,'Algorithm','interior-point','GradObj','on','GradConstr','on','Display','off');

%N=total number of genes. In the example dataset, N=1000.
N=size(readcount_table,1);
%ELTSeq statistic initialization
ELTSeq_sta=zeros(N,1);

%normalization
nfactor=mean(mean(readcount_table))./mean(readcount_table);
for n=1:N
    readcount_table(n,:)=readcount_table(n,:).*nfactor;
end

%run ELTSeq
perc=10;
for n=1:N
    if n/N>=perc/100
        fprintf('%d%% done\n',perc);
        perc=perc+10;
    end
    %filter the lowly expressed genes
    if median(readcount_table(n,1:n1))==0 && median(readcount_table(n,n1+1:n1+n2))==0
        ELTSeq_sta(n)=-Inf;
        continue;
    end
    %deal with extreme case
    if min(readcount_table(n,1:n1))>=max(readcount_table(n,n1+1:n1+n2)) || max(readcount_table(n,1:n1))<=min(readcount_table(n,n1+1:n1+n2))
        ELTSeq_sta(n)=inf;
    else
        %use fmincon function to get test statistics of ELTSeq
        f=@(x)assign(x,readcount_table(n,1:n1),readcount_table(n,n1+1:n1+n2));
        Aeq=[ones(1,n1) zeros(1,n2);zeros(1,n1) ones(1,n2);readcount_table(n,1:n1) -readcount_table(n,n1+1:n1+n2)];
        [xmin,ELTSeq_sta(n)] = fmincon(@negloglike,x0,[],[],Aeq,beq,lb,ub,f,options);
    end
end
%-2log(R) is the final ELTSeq test statistic, which has an asymptotic
%distuibution of Chi-square with d.f. 1.
ELTSeq_sta=2*(ELTSeq_sta-n1*log(n1)-n2*log(n2));
[stat,index]=sort(ELTSeq_sta,'descend');
digits(5);
%calculate asymptotic p-values
sortp=1-chi2cdf(stat,1);
%calculate BH-FDR
fdr=sortp*N./(1:1000)';
fdr(fdr>1)=1;
st=1;
curmin=fdr(1);
for idx=2:N
    if fdr(idx)>curmin
        fdr(st:idx-1)=curmin;
        st=idx;
    end
    curmin=fdr(idx);
end
fdr(st:idx)=curmin;

%% 
%save ELTSeq DE analysis results in a file named 'DE_res.txt'.
res=cell(1,2);
res{2}=[stat sortp fdr];
res{1}=gene_name(index);
fileID = fopen('DE_res.txt','w');
fprintf(fileID,'%10s %10s %10s %10s\n','Gene_name','ELTSeq_stat','p-value','FDR');
formatSpec = '%10s %10.8f %10.5e %10.5e\n';
for row = 1:N
    fprintf(fileID,formatSpec,res{1}{row,:},res{2}(row,:));
end
fclose(fileID);